#ifndef FPUi2c
#define FPUi2c
#include "FPUdefs.bas"

'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file		FPUi2c.bas
' @target:	ARMexpress/ARMmite
'
' This file contains support code for using the uM-FPU V3 floating point
' coprocessor with the ARMexpress or ARMmite over an I2C interface.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	April 2, 2008
'	- updated opcodes in Fpu_ReadFloat
'	August 2, 2007
'	- original version
'==============================================================================

'==============================================================================
'-------------------- uM-FPU V3 I2C definitions -------------------- 2007-03-12
'==============================================================================

const	FpuID		= $C8				' I2C address for FPU
const	FpuPin		= 0					' I2C SDA pin (SCL is FpuPin+1)

'==============================================================================
'-------------------- uM-FPU V3 I2C support routines --------------- 2007-03-12
'==============================================================================

Fpu_Reset:
	I2COUT FpuPin, FpuID, [1, 0]		' send reset command to uM-FPU
	WAIT(10)

	I2COUT FpuPin, FpuID, [0, SYNC]		' check for synchronization
	goto Fpu_Status2

Fpu_Wait:
	I2CIN FpuPin, FpuID, 0, [status]	' wait for ready status
	if status <> 0 then goto Fpu_wait
	return

Fpu_ReadStatus:
	gosub Fpu_Wait						' read status byte
	I2COUT FpuPin, FpuID, [0, READSTATUS]

Fpu_Status2:
	I2CIN FpuPin, FpuID, 0, [status]
	return

Fpu_ReadDelay:
	return								' no extra delay required for I2C

Fpu_ReadString:
	gosub Fpu_Wait						' wait until uM-FPU is ready
	I2COUT FpuPin, FpuID, [0, READSTR]
	gosub Fpu_ReadDelay
	I2CIN FpuPin, FpuID, 0, [str$]
	return

Fpu_WriteFloat:							' write 32-bit floating point
	if reg = -1 then
		I2COUT FpuPin, FpuID, [FWRITEA]
	elseif reg = -2 then
		I2COUT FpuPin, FpuID, [FWRITEX]
	else
		I2COUT FpuPin, FpuID, [FWRITE, reg]
	endif
	goto Fpu_Write32

Fpu_WriteLong:							' write 32-bit integer
	if reg = -1 then
		I2COUT FpuPin, FpuID, [LWRITEA]
	elseif reg = -2 then
		I2COUT FpuPin, FpuID, [LWRITEX]
	else
		I2COUT FpuPin, FpuID, [LWRITE, reg]
	endif
										' (fall through to Fpu_Write32)	
Fpu_Write32:
	I2COUT FpuPin, FpuID, [tmp>>24, tmp>>16, tmp>>8, tmp]
	return

Fpu_ReadFloat:							' read 32-bit floating point
	gosub Fpu_Wait
	if reg = -1 then
		I2COUT FpuPin, FpuID, [FREADA]
	elseif reg = -2 then
		I2COUT FpuPin, FpuID, [FREADX]
	else
		I2COUT FpuPin, FpuID, [FREAD, reg]
	endif
	goto Fpu_Read32
	
Fpu_ReadLong:							' read 32-bit integer
	gosub Fpu_Wait
	if reg = -1 then
		I2COUT FpuPin, FpuID, [LREADA]
	elseif reg = -2 then
		I2COUT FpuPin, FpuID, [LREADX]
	else
		I2COUT FpuPin, FpuID, [LREAD, reg]
	endif
										' (fall through to Fpu_Read32)	
Fpu_Read32:
	gosub Fpu_ReadDelay
	I2CIN FpuPin, FpuID, 0, [tmp, tmp2, tmp3, tmp4]
	tmp = (tmp<<24) + (tmp2<<16) + (tmp3<<8) + tmp4
	return

Fpu_Fcall:
	I2COUT FpuPin, FpuID, [FCALL, tmp]
	return

Print_Version:							' print the uM-FPU version string
	I2COUT FpuPin, FpuID, [0, VERSION]
	goto Print_FpuString

Print_Float:
	format = 0							' set format to zero (free format)
										' (fall through to Print_FloatFormat)
Print_FloatFormat:
	opcode = FTOA						' convert floating point to formatted ASCII
	goto Print_Format

Print_Long:
	format = 0							' set format to 0 (free format)
										' (fall through to Print_LongFormat)
Print_LongFormat:
	opcode = LTOA						' convert long integer to formatted ASCII
										' (fall through to Print_Format)

Print_Format:							' send conversion command
	I2COUT FpuPin, FpuID, [0, opcode, format]

Print_FpuString:						' read string from uM-FPU and print
	gosub Fpu_ReadString
	print str$;
	return

#endif